@@ -1,10 +1,10 @@
 /*
 
-Copyright (c) 2001-2005 Michael Terry
+Copyright (c) 2001-2007 Michael Terry
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
+the Free Software Foundation; either version 3 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
@@ -32,6 +32,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "xpad-text-buffer.h"
 #include "xpad-text-view.h"
 #include "xpad-toolbar.h"
+#include "xpad-tray.h"
 
 G_DEFINE_TYPE(XpadPad, xpad_pad, GTK_TYPE_WINDOW)
 #define XPAD_PAD_GET_PRIVATE(object) (G_TYPE_INSTANCE_GET_PRIVATE ((object), XPAD_TYPE_PAD, XpadPadPrivate))
@@ -79,8 +80,7 @@ enum
   LAST_PROP
 };
 
-static void load_info (XpadPad *pad);
-static void save_info (XpadPad *pad);
+static void load_info (XpadPad *pad, gboolean *show);
 static void load_content (XpadPad *pad);
 static void save_content (XpadPad *pad);
 static GtkWidget *menu_get_popup_highlight (XpadPad *pad, GtkAccelGroup *accel_group);
@@ -108,11 +108,11 @@ static void xpad_pad_show_toolbar (XpadPad *pad);
 static void xpad_pad_popup (XpadPad *pad, GdkEventButton *event);
 static void xpad_pad_spawn (XpadPad *pad);
 static void xpad_pad_clear (XpadPad *pad);
-static void xpad_pad_close (XpadPad *pad);
 static void xpad_pad_delete (XpadPad *pad);
 static void xpad_pad_open_properties (XpadPad *pad);
 static void xpad_pad_open_preferences (XpadPad *pad);
 static void xpad_pad_quit (XpadPad *pad);
+static void xpad_pad_close_all (XpadPad *pad);
 static void xpad_pad_sync_title (XpadPad *pad);
 static void xpad_pad_set_group (XpadPad *pad, XpadPadGroup *group);
 static gboolean xpad_pad_leave_notify_event (GtkWidget *pad, GdkEventCrossing *event);
@@ -130,13 +130,14 @@ xpad_pad_new (XpadPadGroup *group)
 }
 
 GtkWidget *
-xpad_pad_new_with_info (XpadPadGroup *group, const gchar *info_filename)
+xpad_pad_new_with_info (XpadPadGroup *group, const gchar *info_filename, gboolean *show)
 {
 	GtkWidget *pad = GTK_WIDGET (g_object_new (XPAD_TYPE_PAD, "group", group, NULL));
 	
 	XPAD_PAD (pad)->priv->infoname = g_strdup (info_filename);
-	load_info (XPAD_PAD (pad));
+	load_info (XPAD_PAD (pad), show);
 	load_content (XPAD_PAD (pad));
+	gtk_window_set_role (GTK_WINDOW (pad), XPAD_PAD (pad)->priv->infoname);
 	
 	return pad;
 }
@@ -168,7 +169,7 @@ xpad_pad_new_from_file (XpadPadGroup *group, const gchar *filename)
 		g_free (content);
 		
 		g_signal_handlers_unblock_by_func (buffer, xpad_pad_text_changed, pad);
-		xpad_pad_text_changed(pad, buffer);
+		xpad_pad_text_changed(XPAD_PAD(pad), buffer);
 	}
 	
 	return pad;
@@ -221,7 +222,7 @@ xpad_pad_init (XpadPad *pad)
 	pad->priv->height = xpad_settings_get_height (xpad_settings ());
 	pad->priv->infoname = NULL;
 	pad->priv->contentname = NULL;
-	pad->priv->sticky = FALSE;
+	pad->priv->sticky = xpad_settings_get_sticky (xpad_settings ());
 	pad->priv->textview = NULL;
 	pad->priv->scrollbar = NULL;
 	pad->priv->toolbar = NULL;
@@ -254,6 +255,9 @@ xpad_pad_init (XpadPad *pad)
 	g_object_unref (G_OBJECT (accel_group));
 	pad->priv->menu = menu_get_popup_no_highlight (pad, accel_group);
 	pad->priv->highlight_menu = menu_get_popup_highlight (pad, accel_group);
+	gtk_accel_group_connect (accel_group, GDK_Q, GDK_CONTROL_MASK, 0,
+	                         g_cclosure_new_swap (G_CALLBACK (xpad_pad_quit), pad, NULL));
+
 	
 	vbox = GTK_WIDGET (g_object_new (GTK_TYPE_VBOX,
 		"homogeneous", FALSE,
@@ -272,7 +276,7 @@ xpad_pad_init (XpadPad *pad)
 		"skip-taskbar-hint", !xpad_settings_get_has_decorations (xpad_settings ()),
 		"type", GTK_WINDOW_TOPLEVEL,
 		"type-hint", GDK_WINDOW_TYPE_HINT_NORMAL,
-		"window-position", GTK_WIN_POS_NONE,
+		"window-position", GTK_WIN_POS_MOUSE,
 		"child", vbox,
 		NULL);
 	
@@ -306,7 +310,7 @@ xpad_pad_init (XpadPad *pad)
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-delete", G_CALLBACK (xpad_pad_delete), pad);
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-properties", G_CALLBACK (xpad_pad_open_properties), pad);
 	g_signal_connect_swapped (pad->priv->toolbar, "activate-preferences", G_CALLBACK (xpad_pad_open_preferences), pad);
-	g_signal_connect_swapped (pad->priv->toolbar, "activate-quit", G_CALLBACK (xpad_pad_quit), pad);
+	g_signal_connect_swapped (pad->priv->toolbar, "activate-quit", G_CALLBACK (xpad_pad_close_all), pad);
 	
 	g_signal_connect (pad->priv->toolbar, "popup", G_CALLBACK (xpad_pad_toolbar_popup), pad);
 	g_signal_connect (pad->priv->toolbar, "popdown", G_CALLBACK (xpad_pad_toolbar_popdown), pad);
@@ -314,7 +318,7 @@ xpad_pad_init (XpadPad *pad)
 	g_signal_connect (pad->priv->menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
 	g_signal_connect (pad->priv->highlight_menu, "deactivate", G_CALLBACK (xpad_pad_popup_deactivate), pad);
 	
-	if (xpad_settings_get_sticky (xpad_settings ()))
+	if (pad->priv->sticky)
 		gtk_window_stick (GTK_WINDOW (pad));
 	else
 		gtk_window_unstick (GTK_WINDOW (pad));
@@ -344,14 +348,14 @@ xpad_pad_show (XpadPad *pad)
 		"gravity", GDK_GRAVITY_STATIC,
 		"skip-pager-hint", TRUE,
 		"skip-taskbar-hint", TRUE,
-		NULL);
+		NULL);*/
 	
-	if (xpad_settings_get_sticky (xpad_settings ()))
+	if (pad->priv->sticky)
 		gtk_window_stick (GTK_WINDOW (pad));
 	else
 		gtk_window_unstick (GTK_WINDOW (pad));
 	
-	xpad_pad_sync_title (pad);*/
+/*	xpad_pad_sync_title (pad);*/
 }
 
 static void
@@ -414,7 +418,6 @@ xpad_pad_notify_has_scrollbar (XpadPad *pad)
 static void
 xpad_pad_notify_has_decorations (XpadPad *pad)
 {
-	gboolean shown = GTK_WIDGET_VISIBLE (GTK_WIDGET (pad));
 	gboolean decorations = xpad_settings_get_has_decorations (xpad_settings ());
 	
 	/**
@@ -456,9 +459,6 @@ xpad_pad_show_toolbar (XpadPad *pad)
 	if (!GTK_WIDGET_VISIBLE (pad->priv->toolbar))
 	{
 		GtkRequisition req;
-		GdkRectangle rec;
-		gint textx, texty, x, y;
-		GtkTextIter iter;
 		
 		if (GTK_WIDGET (pad)->window)
 			gdk_window_freeze_updates (GTK_WIDGET (pad)->window);
@@ -491,10 +491,6 @@ xpad_pad_hide_toolbar (XpadPad *pad)
 {
 	if (GTK_WIDGET_VISIBLE (pad->priv->toolbar))
 	{
-		GdkRectangle rec;
-		gint textx, texty, x, y;
-		GtkTextIter iter;
-		
 		if (GTK_WIDGET (pad)->window)
 			gdk_window_freeze_updates (GTK_WIDGET (pad)->window);
 		gtk_widget_hide (pad->priv->toolbar);
@@ -596,15 +592,25 @@ xpad_pad_clear (XpadPad *pad)
 	gtk_text_buffer_set_text (buffer, "", -1);
 }
 
-static void
+void
 xpad_pad_close (XpadPad *pad)
 {
 	gtk_widget_hide (GTK_WIDGET (pad));
 	
+	/* If no tray and this is the last pad, we don't want to record this
+	   pad as closed, we want to start with just this pad next open.  So
+	   quit before we record. */
+	if (!xpad_tray_is_open () &&
+	    xpad_pad_group_num_visible_pads (pad->priv->group) == 0)
+	{
+		xpad_pad_quit (pad);
+		return;
+	}
+	
 	if (pad->priv->properties)
 		gtk_widget_destroy (pad->priv->properties);
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 	
 	g_signal_emit (pad, signals[CLOSED], 0);
 }
@@ -705,7 +711,7 @@ prop_notify_follow_font (XpadPad *pad)
 			pango_font_description_free (fontdesc);
 	}
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 }
 
 static void
@@ -721,7 +727,7 @@ prop_notify_follow_color (XpadPad *pad)
 		gtk_widget_modify_text (pad->priv->textview, GTK_STATE_NORMAL, xpad_pad_properties_get_text_color (prop));
 	}
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 }
 
 static void
@@ -731,7 +737,7 @@ prop_notify_text (XpadPad *pad)
 	
 	gtk_widget_modify_text (pad->priv->textview, GTK_STATE_NORMAL, xpad_pad_properties_get_text_color (prop));
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 }
 
 static void
@@ -741,7 +747,7 @@ prop_notify_back (XpadPad *pad)
 	
 	gtk_widget_modify_base (pad->priv->textview, GTK_STATE_NORMAL, xpad_pad_properties_get_back_color (prop));
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 }
 
 static void
@@ -757,7 +763,7 @@ prop_notify_font (XpadPad *pad)
 	if (fontdesc)
 		pango_font_description_free (fontdesc);
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 }
 
 static void
@@ -834,6 +840,9 @@ xpad_pad_toolbar_size_allocate (XpadPad *pad, GtkAllocation *event)
 static gboolean
 xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 {
+	if (!GTK_WIDGET_VISIBLE (pad))
+		return FALSE;
+	
 	if (pad->priv->width != event->width || pad->priv->height != event->height)
 		pad->priv->toolbar_pad_resized = TRUE;
 	
@@ -843,7 +852,7 @@ xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 	pad->priv->height = event->height;
 	pad->priv->location_valid = TRUE;
 	
-	save_info (pad);
+	xpad_pad_save_info (pad);
 	
 	/* Sometimes when moving, if the toolbar tries to hide itself,
 	   the window manager will not resize it correctly.  So, we make
@@ -860,10 +869,13 @@ xpad_pad_configure_event (XpadPad *pad, GdkEventConfigure *event)
 static gboolean
 xpad_pad_window_state_event (XpadPad *pad, GdkEventWindowState *event)
 {
-	pad->priv->sticky = (event->new_window_state & GDK_WINDOW_STATE_STICKY) ? TRUE : FALSE;
-	
-	if (event->changed_mask & GDK_WINDOW_STATE_STICKY)
-		save_info (pad);
+	if (event->changed_mask & GDK_WINDOW_STATE_STICKY) {
+		if (GTK_WIDGET_VISIBLE (pad))
+		{
+			pad->priv->sticky = (event->new_window_state & GDK_WINDOW_STATE_STICKY) ? TRUE : FALSE;
+			xpad_pad_save_info (pad);
+		}
+	}
 	
 	return FALSE;
 }
@@ -1072,7 +1084,7 @@ save_content (XpadPad *pad)
 }
 
 static void
-load_info (XpadPad *pad)
+load_info (XpadPad *pad, gboolean *show)
 {
 	gboolean locked = FALSE, follow_font = TRUE, follow_color = TRUE;
 	gboolean hidden = FALSE;
@@ -1157,12 +1169,12 @@ load_info (XpadPad *pad)
 	}
 	g_free (oldcontentprefix);
 	
-	if (hidden)
-		gtk_widget_hide (GTK_WIDGET (pad));
+	if (show)
+		*show = !hidden;
 }
 
-static void
-save_info (XpadPad *pad)
+void
+xpad_pad_save_info (XpadPad *pad)
 {
 	gint height;
 	GtkStyle *style;
@@ -1174,6 +1186,7 @@ save_info (XpadPad *pad)
 		pad->priv->infoname = fio_unique_name ("info-");
 		if (!pad->priv->infoname)
 			return;
+		gtk_window_set_role (GTK_WINDOW (pad), pad->priv->infoname);
 	}
 	/* create content file if it doesn't exist yet */
 	if (!pad->priv->contentname)
@@ -1198,7 +1211,7 @@ save_info (XpadPad *pad)
 		"b|follow_font", xpad_text_view_get_follow_font_style (XPAD_TEXT_VIEW (pad->priv->textview)),
 		"b|follow_color", xpad_text_view_get_follow_color_style (XPAD_TEXT_VIEW (pad->priv->textview)),
 		"b|sticky", pad->priv->sticky,
-		"b|hidden", GTK_WIDGET_VISIBLE (pad),
+		"b|hidden", !GTK_WIDGET_VISIBLE (pad),
 		"h|back_red", style->base[GTK_STATE_NORMAL].red,
 		"h|back_green", style->base[GTK_STATE_NORMAL].green,
 		"h|back_blue", style->base[GTK_STATE_NORMAL].blue,
@@ -1218,12 +1231,12 @@ menu_about (XpadPad *pad)
 	const gchar *artists[] = {"Michael Terry <mike@mterry.name>", NULL};
 	const gchar *authors[] = {"Michael Terry <mike@mterry.name>", "Jeroen Vermeulen <jtv@xs4all.nl>", NULL};
 	const gchar *comments = _("Sticky notes");
-	const gchar *copyright = "© 2001-2006 Michael Terry";
+	const gchar *copyright = "© 2001-2007 Michael Terry";
 	/* we use g_strdup_printf because C89 has size limits on static strings */
 	gchar *license = g_strdup_printf ("%s\n%s\n%s",
 "This program is free software; you can redistribute it and/or\n"
 "modify it under the terms of the GNU General Public License\n"
-"as published by the Free Software Foundation; either version 2\n"
+"as published by the Free Software Foundation; either version 3\n"
 "of the License, or (at your option) any later version.\n"
 ,
 "This program is distributed in the hope that it will be useful,\n"
@@ -1301,6 +1314,24 @@ menu_show_all (XpadPad *pad)
 	g_slist_free (pads);
 }
 
+static void
+xpad_pad_close_all (XpadPad *pad)
+{
+	if (!pad->priv->group)
+		return;
+	
+	/**
+	 * The logic is different here depending on whether the tray is open.
+	 * If it is open, we just close each pad individually.  If it isn't
+	 * open, we do a quit.  This way, when xpad is run again, only the
+	 * pads open during the last 'close all' will open again.
+	 */
+	if (xpad_tray_is_open ())
+		xpad_pad_group_close_all (pad->priv->group);
+	else
+		xpad_pad_quit (pad);
+}
+
 static void
 menu_show (XpadPad *pad)
 {
@@ -1467,6 +1498,7 @@ menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 	MENU_ADD_STOCK (GTK_STOCK_NEW, xpad_pad_spawn);
 	MENU_ADD_SEP ();
 	MENU_ADD_CHECK (_("Show on _All Workspaces"), pad->priv->sticky, menu_sticky);
+	g_object_set_data (G_OBJECT (uppermenu), "sticky", item);
 	MENU_ADD_STOCK (GTK_STOCK_PROPERTIES, xpad_pad_open_properties);
 	MENU_ADD_SEP ();
 	MENU_ADD_STOCK (GTK_STOCK_CLOSE, xpad_pad_close);
@@ -1509,7 +1541,7 @@ menu_get_popup_no_highlight (XpadPad *pad, GtkAccelGroup *accel_group)
 	g_object_set_data (G_OBJECT (uppermenu), "notes-menu", menu);
 	
 	MENU_ADD (_("_Show All"), NULL, 0, 0, menu_show_all);
-	MENU_ADD (_("_Close All"), GTK_STOCK_QUIT, GDK_q, GDK_CONTROL_MASK, xpad_pad_quit);
+	MENU_ADD (_("_Close All"), NULL, 0, 0, xpad_pad_close_all);
 	
 	/* The rest of the notes menu will get set up in the prep function below */
 	
@@ -1538,6 +1570,13 @@ menu_prep_popup_no_highlight (XpadPad *current_pad, GtkWidget *uppermenu)
 	if (item)
 		gtk_widget_set_sensitive (item, gtk_clipboard_wait_is_text_available (clipboard));
 	
+	item = g_object_get_data (G_OBJECT (uppermenu), "sticky");
+	if (item) {
+		g_signal_handlers_block_by_func (item, menu_sticky, current_pad);
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), current_pad->priv->sticky);
+		g_signal_handlers_unblock_by_func (item, menu_sticky, current_pad);
+	}
+	
 	menu = g_object_get_data (G_OBJECT (uppermenu), "notes-menu");
 	if (menu)
 	{
